import com.dalsemi.system.BitPort;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * DeviceController servlet.
 *
 * Shows how a system can display a Web page and respond when a user
 * clicks a link that sends information to the servlet.
 *
 * The Web page enables users to monitor and control two LEDs on a TINI.
 *
 * Tested with the Tynamo Web server (www.tynamo.com) and
 * DSTINIs400 TINI module (www.dalsemi.com).
 *
 * The servlet's Web page contains links to the following image files:
 *   ledon.gif
 *   ledoff.gif
 *   button.gif
 *
 * Before deploying the servlet to the TINI, YOU MUST do the following:
 *
 * If using the default webserver.props file, store the image files in the
 * http-root directory under Tynamo's home directory on the development
 * computer. (Or edit webserver.props to specify a different directory and store
 * the files there.)
 *
 * Edit the provided build.properties file to include the locations of this
 * DeviceController.java file and the TINI installation on your development
 * computer.
 *
 * For more on compiling the servlet and deploying it to a TINI,
 * see www.tynamo.com.
 *
 * This file and other embedded Ethernet and Internet code and resources are
 * available from www.Lvr.com.
 *
 * @author Jan Axelson
 * @author Shawn Silverman
 */

public class DeviceController extends HttpServlet
{
  // led1 is included on the DSTINIm400 board.
  // led2 is an optional LED to be added and controlled
  // in a similar way at port 5, bit 3.
  BitPort led1 = new BitPort(BitPort.Port5Bit4);
  BitPort led2 = new BitPort(BitPort.Port5Bit5);

  /**
   * The initial state for the LEDS is off.
   *
   * @throws ServletException
   */
  public void init() throws ServletException {

    // A logic high turns the corresponding LED off.
    led1.set();
    led2.set();
  } // end init

  /**
   * Responds to HTTP GET requests.
   *
   * @param request the HTTPServletRequest object
   * @param response the HttpServletResponse object
   *
   * @throws ServletException
   * @throws IOException
   */
  public void doGet(HttpServletRequest request, HttpServletResponse response)
      throws ServletException, IOException
  {
    //The query string sent by the client tells which button was clicked
    // on the Web page.
    String query = request.getQueryString();

    // Read the current state of the LEDs (0=on, 1=off).
    // and toggle the LED named in the query string.
    boolean led1On;
    if ("button1".equals(query)) {
      System.out.println("Button 1 was clicked");

      // Toggle the LED.
      led1On = toggle(led1);
    } else {

      // Don't toggle the LED.
      // Read the LED's state and set led1On true if the state is 0
      // (on) and false if the state is 1 (off).
      led1On = (led1.readLatch() == 0);
    }

    boolean led2On;
    if ("button2".equals(query)) {
      System.out.println("Button 2 was clicked");

      // Toggle the LED.
      // A logic low turns the LED on.
      led2On = toggle(led2);
    } else {

      // Don't toggle the LED.
      // Read the LED's state and set led2On true if the state is 0
      // (on) and false if the state is 1 (off).
      led2On = (led2.readLatch() == 0);
    }

    // Set the images and text to match the current LEDs' states
    // If led1State (led2State) is true, LED1 (LED2) is off.
    // If led2State (led2State) is false, LED1 (LED2) is on.
    String led1Image;
    String led1State;
    if (led1On) {
      led1Image= "/ledon.gif";
      led1State = "on";
    }
    else {
      led1Image = "/ledoff.gif";
      led1State = "off";
    }

    String led2Image;
    String led2State;
    if (led2On) {
      led2Image= "/ledon.gif";
      led2State = "on";
    }
    else {
      led2Image = "/ledoff.gif";
      led2State = "off";
    }

    // Return the Web page to the client.
    SendWebPage (response, led1Image, led2Image,
                 led1State, led2State);

  } //end doGet

  /**
   * Toggles the specified bitPort bit and returns the new state.
   *
   * @param bitPort a bit that controls an LED.
   */

  private static boolean toggle(BitPort bitPort) {
    if (bitPort.readLatch() == 0) {

      // If it's 0, set the bit to turn the LED off and return false.
      bitPort.set();
      return false;
    } else {

      // If it's 1, clear the bit to turn the LED on and return true.
      bitPort.clear();
      return true;
    }
  } // end toggle

  /**
   * Sends a Web page to the output stream.
   *
   * @param response the HttpServletResponse object
   * @param led1Image name of an image file
   * @param led2Image name of an image file
   * @param led1State "on" or "off"
   * @param led2State "on" or "off"
   *
   * @throws IOException
   */
  private void SendWebPage (HttpServletResponse response,
                            String led1Image, String led2Image,
                            String led1State, String led2State)
      throws IOException {

    // Write the Web page to the client.
    // Add an escape character (\) before all quotes (") within the HTML code
    // (but not before the quotes required at the beginning and end
    // of each out.print statement).
    // When writing to the output stream, concatenate string constants with "+",
    // to minimize the number of writes to the output stream while keeping the code
    // readable and because there is no performance penalty.

    // Set the Content-Type field in the HTML header.
    response.setContentType("text/html");

    //Use a ServletOutputStream object to write the page.
    ServletOutputStream out = response.getOutputStream();

    // Write the page's contents to the output stream.
    // Place the LED and button images in a table so they line up.
    // The variables led1Image and led2Image each contain a file name,
    // either "ledon.gif" or "ledoff.gif", as appropriate.
    out.print("<html>"
              + "<head><title>Device Controller</title></head>"
              + "<body>"
              + "<h1>Device Controller</h1>"
              + "<table><tr><td>"
              +"<img src=\"");
    out.print(led1Image);
    out.print("\">"
              + "</td>"
              + "<td>"
              + "<img src=\"");
    out.print(led2Image);
    out.print("\">"
              + "</td>"
              + "</tr>");

    // /servlet/DeviceController is the mapping that identifies DeviceController
    // as the DeviceController servlet, as specified in the servlets.props file.
    // button1 or button2 is the query string returned to the TINI
    // when the user clicks a button.
    out.print("<tr>"
              + "<td>"
              + "<a href=\"/servlet/DeviceController?button1\"><img SRC=\"/button.gif\"></a>"
              + "</td>"
              + "<td>"
              + "<a href=\"/servlet/DeviceController?button2\"><img SRC=\"/button.gif\" ></a>"
              + "</td>"
              + "</tr>"
              + "</table>"
              + "<p>LED 1 is ");
    out.print (led1State);

    // Send text that matches the LEDs' states.
    out.print(".</p>"
              + "<p>LED 2 is ");
    out.print (led2State);
    out.print(".</p>"
              + "<p>Click a button to turn an LED on or off.</p>"
              + "<p>The web page will update to show the current states of the LEDs.</p>"
              + "</body>"
              + "</html>");
    // The servlet container closes the output stream when the request has been
    // serviced, so there's no need to do it here.
  } // end SendWebPage

  /**
   * Gets information about this servlet.
   */
  public String getServletInfo()
  {
    return "Device Controller servlet";
  } // end getServletInfo

} // end DeviceController servlet